# imports dataclass which is used to create datatypes for dice and state
from dataclasses import dataclass


ACES   = "ACES"
TWOS   = "TWOS"
THREES = "THREES"
FOURS  = "FOURS"
FIVES  = "FIVES"
SIXES  = "SIXES"
SEVENS = "SEVENS"
EIGHTS = "EIGHTS"
NINES  = "NINES"
TENS   = "TENS"
JACKS  = "JACKS"
QUEENS = "QUEENS"
KINGS  = "KINGS"

FACES = [ACES, TWOS, THREES, FOURS, FIVES, SIXES, SEVENS, EIGHTS, NINES, TENS, JACKS, QUEENS, KINGS]

ACES_STRING   = "A"
TWOS_STRING   = "2"
THREES_STRING = "3"
FOURS_STRING  = "4"
FIVES_STRING  = "5"
SIXES_STRING  = "6"
SEVENS_STRING = "7"
EIGHTS_STRING = "8"
NINES_STRING  = "9"
TENS_STRING   = "T"
JACKS_STRING  = "J"
QUEENS_STRING = "Q"
KINGS_STRING  = "K"

FACES_STRING = [ACES_STRING, TWOS_STRING, THREES_STRING, FOURS_STRING, FIVES_STRING, SIXES_STRING, SEVENS_STRING, EIGHTS_STRING, NINES_STRING, TENS_STRING, JACKS_STRING, QUEENS_STRING, KINGS_STRING]


@dataclass
class Face:
    value: str


    def __post_init__(self):
        if type(self.value) == str:
            
            if self.value in FACES:
                pass

            else:
                raise ValueError()

        else:
            raise TypeError()


    @classmethod
    def string_to_face(cls, face_string: str):
        if type(face_string) == str:

            if face_string == ACES_STRING:
                return Face(ACES)

            elif face_string == TWOS_STRING:
                return Face(TWOS)

            elif face_string == THREES_STRING:
                return Face(THREES)

            elif face_string == FOURS_STRING:
                return Face(FOURS)

            elif face_string == FIVES_STRING:
                return Face(FIVES)

            elif face_string == SIXES_STRING:
                return Face(SIXES)

            elif face_string == SEVENS_STRING:
                return Face(SEVENS)

            elif face_string == EIGHTS_STRING:
                return Face(EIGHTS)

            elif face_string == NINES_STRING:
                return Face(NINES)

            elif face_string == TENS_STRING:
                return Face(TENS)

            elif face_string == JACKS_STRING:
                return Face(JACKS)

            elif face_string == QUEENS_STRING:
                return Face(QUEENS)

            elif face_string == KINGS_STRING:
                return Face(KINGS)

            else:
                raise ValueError()
                    
        else:
            TypeError()


    @classmethod
    def face_to_string(cls, face):
        if type(face) == Face:

            if face.value == ACES:
                return ACES_STRING 

            elif face.value == TWOS: 
                return TWOS_STRING

            elif face.value == THREES:
                return THREES_STRING

            elif face.value == FOURS: 
                return FOURS_STRING

            elif face.value == FIVES: 
                return FIVES_STRING

            elif face.value == SIXES:
                return SIXES_STRING

            elif face.value == SEVENS: 
                return SEVENS_STRING

            elif face.value == EIGHTS: 
                return EIGHTS_STRING

            elif face.value == NINES:
                return NINES_STRING

            elif face.value == TENS:
                return TENS_STRING

            elif face.value == JACKS:
                return JACKS_STRING

            elif face.value == QUEENS:
                return QUEENS_STRING

            elif face.value == KINGS:
                return KINGS_STRING

            else:
                raise ValueError()
             
        else:
            TypeError()


    def __str__(self):
        return Face.face_to_string(self)


def test():
    for i in range(len(FACES)):
        face = FACES[i]
        face_string = FACES_STRING[i]
        f = Face(face)
        print(face)
        print(f.value == face)
        print(str(f) == face_string)
        print(f.value == Face.string_to_face(Face.face_to_string(f)).value)
        print(face_string == Face.face_to_string(Face.string_to_face(face_string)))


if __name__ == "__main__":
    test()



# imports random module for random number generation for random dice values
import random

# imports dataclass which is used to create datatypes for dice and state
from dataclasses import dataclass

# imports typing which are type annotations
from typing import Any
from typing import Union
from typing import List
from typing import Tuple

# imports copy which is used to deepcopy a value
import copy

# imports suit which is used to store card suits
import suit as _suit

# imports face which is used to store card faces
import face as _face


@dataclass
class Card:
    suit: _suit.Suit
    face: _face.Face


    def __post_init__(self):
        if type(self.suit) == _suit.Suit:
            
            if type(self.face) == _face.Face:
                pass

            else:
                raise TypeError()

        else:
            raise TypeError()


    @classmethod
    def string_to_card(cls, card_string: str):
        if type(card_string) == str:
            suit_string, face_string = card_string.split()

            return Card(suit=_suit.Suit.string_to_suit(suit_string), face=_face.Face.string_to_face(face_string))

        else:
            TypeError()


    @classmethod
    def card_to_string(cls, card):
        if type(card) == Card:
            return str(card.suit) + " " + str(card.face)
                
        else:
            TypeError()


    def __str__(self):
        return Card.card_to_string(self) 


def test():
    for i in range(len(_face.FACES)):
        face = _face.FACES[i]
        face_string = _face.FACES_STRING[i]
        f = _face.Face(face)

        for j in range(len(_suit.SUITS)):
            suit = _suit.SUITS[j]
            suit_string = _suit.SUITS_STRING[j]
            s = _suit.Suit(suit)

            card = Card(s, f)
            card2 = Card.string_to_card(Card.card_to_string(card))

            print(card)
            print(card2.face.value == card.face.value and card2.suit.value == card.suit.value)
            print(Card.card_to_string(Card.string_to_card(str(card))) == str(card))
            print(card2.face.value == face and card2.suit.value == suit)


if __name__ == "__main__":
    test()      



# imports dataclass which is used to create datatypes for dice and state
from dataclasses import dataclass


HEARTS   = "HEARTS"
CLUBS    = "CLUBS"
SPADES   = "SPADES"
DIAMONDS = "DIAMONDS"

SUITS = [HEARTS, CLUBS, SPADES, DIAMONDS]

HEARTS_STRING   = "H"
CLUBS_STRING    = "C"
SPADES_STRING   = "S"
DIAMONDS_STRING = "D"

SUITS_STRING = [HEARTS_STRING, CLUBS_STRING, SPADES_STRING, DIAMONDS_STRING]


@dataclass
class Suit:
    value: str


    def __post_init__(self):
        if type(self.value) == str:
            
            if self.value in SUITS:
                pass

            else:
                raise ValueError()

        else:
            raise TypeError()


    @classmethod
    def string_to_suit(cls, suit_string: str):
        if type(suit_string) == str:

            if suit_string == HEARTS_STRING:
                return Suit(HEARTS)

            elif suit_string == CLUBS_STRING:
                return Suit(CLUBS)

            elif suit_string == SPADES_STRING:
                return Suit(SPADES)

            elif suit_string == DIAMONDS_STRING:
                return Suit(DIAMONDS)

            else:
                raise ValueError()

        else:
            TypeError()


    @classmethod
    def suit_to_string(cls, suit):
        if type(suit) == Suit:

            if suit.value == HEARTS:
                return HEARTS_STRING

            elif suit.value == CLUBS:
                return CLUBS_STRING

            elif suit.value == SPADES:
                return SPADES_STRING

            elif suit.value == DIAMONDS:
                return DIAMONDS_STRING

            else:
                raise ValueError()
                
        else:
            TypeError()


    def __str__(self):
        return Suit.suit_to_string(self)


def test():
    
    for i in range(len(SUITS)):
        suit = SUITS[i]
        suit_string = SUITS_STRING[i]

        s = Suit(suit)

        print(suit)
        print(s.value == suit)

        # str test
        print(str(s) == suit_string)

        # suit_to_string test and string_to_suit
        print(s.value == Suit.string_to_suit(Suit.suit_to_string(s)).value)
        print(suit_string == Suit.suit_to_string(Suit.string_to_suit(suit_string)))


if __name__ == "__main__":
    test()